@@ -54,5 +54,6 @@ pub fn print_test_results(results: &[TestResult]) {
     let _ = stdout.set_color(&status_color);
     writeln!(&mut stdout, "{}/{} Passed", passed, total).unwrap();
     // Reset the color settings back to what the user configured
-    let _ = stdout.reset();
+    let _ = stdout.set_color(&ColorSpec::new());
+    writeln!(&mut stdout, "").unwrap();
 }
